#include <owl\pch.h>
#include <windows.h>
#include <owl\button.h>
#include <owl\checkbox.h>
#include <owl\dialog.h>
#include <owl\edit.h>
#include <owl\combobox.h>
#include <owl\listbox.h>
#include <owl\listwind.h>
#include <owl\slider.h>
#include <owl\opensave.h>
#include <owl\imagelst.h>

#include "midasdll.h"
#include "mem.h"
#include "string.h"

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <shellapi.h>

#include <modality.h>
#include <midasplayer.h>
#pragma hdrstop
#pragma warn -aus


lpMIDASPlayer midas;

/******************************************************************************/
// TPrefDlg Interface
/******************************************************************************/

class TPrefDlg : public TDialog
{
   public:
    TPrefDlg(TWindow* parent, TResId resId, TModule* module = 0);
    ~TPrefDlg();
    virtual void SetupWindow();
    void CmOk();
    void SetupGUI();

    TComboBox *cb_Quality, *cb_Mixrate, *cb_Filter, *cb_BSizes, *cb_BBuffers;
    TCheckBox *cb_LoadNames, *cb_BadFiles, *cb_IconTNA, *cb_PlayBW;
    TStatic *st_Important;

    TFont *f;
   DECLARE_RESPONSE_TABLE(TPrefDlg);
};

DEFINE_RESPONSE_TABLE1(TPrefDlg, TDialog)
 EV_BN_CLICKED(1, CmOk),
END_RESPONSE_TABLE;

/******************************************************************************/
// TInstrDialog Interface
/******************************************************************************/
class TInstrDialog : public TDialog
{
  public:
   TListBox *InstrList;

   TFont f;
   TInstrDialog(TWindow* parent = 0, TResId resId ="DLG_INSTRUMENTS", TModule* module = 0);

   void UpdateInstrList(MIDASmodule module);
   virtual void SetupWindow();
};

/******************************************************************************/
// TInstrFrameWindow Interface
/******************************************************************************/
class TInstrFrameWindow : public TFrameWindow
{
  public:
   TInstrFrameWindow (TWindow* parent, const char far *title = APP_TITLE,
                   TWindow *clientWnd = 0, BOOL shrinkToClient = true)
                  :TFrameWindow (parent, title, clientWnd, shrinkToClient)
   {
     Attr.Style = WS_POPUPWINDOW &~WS_THICKFRAME &~WS_MAXIMIZEBOX &~WS_CAPTION;
     Attr.ExStyle |= WS_EX_DLGMODALFRAME;
   }
};

/******************************************************************************/
// TDiskDialog Interface
/******************************************************************************/
class TDiskDialog : public TDialog
{
  public:
   TListWindow *FileList;
   TImageList *img;

   int cntxt_idx;


   char module[MAX_PATH];

   TDiskDialog(TWindow* parent = 0, TResId resId ="DLG_DISK", TModule* module = 0);
   virtual void SetupWindow();
   virtual void CleanupWindow();

   int LoadModulelist(int c = 1);
   int SaveModulelist();
   int SetupGUI();
   int AddNewFiles();
   int PlayIndex(int i);
   int SetupNewModule(int cmd = CMD_AUTO);
   int AddFile(char *p, char *fn);
   int MarkModule(int idx, int b);

   void EvModuleDblclk();
   void EvContextMenu(HWND child, int x, int y);
   void CmPlay();
   void CmDelete();
   void CmAddFile();
   void CmLstNew();
   void CmLstOpen();
   void CmLstSave();
   void CmLstSaveAs();
   void UnloadModule();

   void EvDropFiles(TDropInfo dropInfo);
   void ProcessCmdLine();
   void LoadModulelistWithCmdLineCheck();
   void UpdateTitle();
   int FileExists(char *p);

   DECLARE_RESPONSE_TABLE(TDiskDialog);
};

DEFINE_RESPONSE_TABLE1(TDiskDialog, TWindow)
 EV_NM_DBLCLK(100, EvModuleDblclk),
 EV_WM_CONTEXTMENU,
 EV_COMMAND(2100, CmPlay),
 EV_COMMAND(2102, CmDelete),
 EV_COMMAND(2101, CmAddFile),
 EV_COMMAND(2103, CmLstNew),
 EV_COMMAND(2104, CmLstOpen),
 EV_COMMAND(2105, CmLstSave),
 EV_COMMAND(2106, CmLstSaveAs),
 EV_WM_DROPFILES,

END_RESPONSE_TABLE;

/******************************************************************************/
// TDiskFrameWindow Interface
/******************************************************************************/
class TDiskFrameWindow : public TFrameWindow
{
  public:
   TDiskFrameWindow (TWindow* parent, const char far *title = APP_TITLE,
                   TWindow *clientWnd = 0, BOOL shrinkToClient = true)
                  :TFrameWindow (parent, title, clientWnd, shrinkToClient)
   {
     Attr.Style = WS_POPUPWINDOW &~WS_THICKFRAME &~WS_MAXIMIZEBOX &~WS_CAPTION;
     Attr.ExStyle |= WS_EX_DLGMODALFRAME;
   }
};

/******************************************************************************/
// TMainDialog Interface
/******************************************************************************/
class TMainDialog : public TDialog
{
  public:

  enum TModuleState {nothing, loaded, playing};

  MIDASmodule module;
  MIDASmodulePlayHandle pmodule;
  MIDASmoduleInfo imodule;
  MIDASplayStatus pstatus;

  TModuleState ms;
  char fname[MAX_PATH];

  TStatic *st_title, *st_fname, *st_type, *st_arr, *st_pat, *st_chan, *st_instr;
  THSlider *sl_pos;
  TVSlider *sl_vol;
  TButton *bt_play, *bt_stop, *bt_forward, *bt_rewind;
  TCheckBox *cb_repeat, *cb_shuffle, *cb_repeat2, *cb_disk, *cb_instr;

  TIcon *icoSmall, *icoNothing, *icoPlay;
  TFont f;
  TPopupMenu *pop;
  HANDLE hMailslot;

  signed int lastpos;

  TMainDialog(TWindow* parent = 0, TResId resId ="DLG_MAIN", TModule* module = 0);
  ~TMainDialog();

  void CmPlay();
  void CmStop();
  void CmForward();
  void CmRewind();
  void CmFForward();
  void CmFRewind();
  void CmFAuto();
  void CmRepeat();
  void CmRepeat2();
  void CmDisk();
  void CmInstr();
  void CmShuffle();
  void CmMnuShuffle();
  void CmMnuRepeat1();
  void CmMnuRepeat();
  void CmAbout();
  void CmActivate();
  void CmPref();
  void CmMnuDisk();
  void CmMnuInstr();
  void CmMnuExit();
  void CmLstNew();
  void CmLstOpen();
  void CmLstSave();
  void CmLstSaveAs();
  void CmMnuHelp();

  virtual void SetupWindow();
  virtual void CleanupWindow();

  int LoadFile();
  int UnloadFile(int upd = 1);
  int PlayModule(unsigned n);
  int StopModule(int upd = 1);
  int ShowStuckInfo();
  int ShowVarInfo();
  int SetupOptions();
  int InitMIDAS();
  int DoneMIDAS();
  int SetupGUI();
  int UpdateState();

  void ProcessMailSlot();
  void InitMailSlot();
  void CloseMailSlot();
  void UpdateFlags();

  void EvTimer(uint timerId);
  void EvTrack();
  void EvVol();
  LRESULT EvTNANotify(WPARAM, LPARAM);
  LRESULT EvAfterInit(WPARAM, LPARAM);

  DECLARE_RESPONSE_TABLE(TMainDialog);
};

DEFINE_RESPONSE_TABLE1(TMainDialog, TDialog)
 EV_MESSAGE(WM_AFTERINIT, EvAfterInit),
 EV_MESSAGE(WM_TNANOTIFY, EvTNANotify),
 EV_BN_CLICKED(107, CmPlay),
 EV_BN_CLICKED(108, CmStop),
 EV_BN_CLICKED(112, CmForward),
 EV_BN_CLICKED(111, CmRewind),
 EV_BN_CLICKED(110, CmFForward),
 EV_BN_CLICKED(109, CmFRewind),
 EV_BN_CLICKED(118, CmRepeat),
 EV_BN_CLICKED(119, CmShuffle),
 EV_BN_CLICKED(122, CmRepeat2),
 EV_BN_CLICKED(120, CmDisk),
 EV_BN_CLICKED(121, CmInstr),
 EV_WM_TIMER,

 EV_SB_BEGINTRACK(115, EvTrack),
 EV_SB_BOTTOM(115, EvTrack),
 EV_SB_ENDSCROLL(115, EvTrack),
 EV_SB_LINEDOWN(115, EvTrack),
 EV_SB_LINEUP(115, EvTrack),
 EV_SB_PAGEDOWN(115, EvTrack),
 EV_SB_PAGEUP(115, EvTrack),
 EV_SB_THUMBPOSITION(115, EvTrack),
 EV_SB_TOP(115, EvTrack),

 EV_SB_BEGINTRACK(117, EvVol),
 EV_SB_BOTTOM(117, EvVol),
 EV_SB_ENDSCROLL(117, EvVol),
 EV_SB_LINEDOWN(117, EvVol),
 EV_SB_LINEUP(117, EvVol),
 EV_SB_PAGEDOWN(117, EvVol),
 EV_SB_PAGEUP(117, EvVol),
 EV_SB_THUMBPOSITION(117, EvVol),
 EV_SB_TOP(117, EvVol),

 EV_COMMAND(3106, CmPref),
 EV_COMMAND(3101, CmFForward),
 EV_COMMAND(3102, CmFRewind),
 EV_COMMAND(3103, CmFAuto),

 EV_COMMAND(3104, CmMnuShuffle),
 EV_COMMAND(3105, CmMnuRepeat1),
 EV_COMMAND(3108, CmMnuRepeat),
 EV_COMMAND(1103, CmMnuHelp),
 EV_COMMAND(3107, CmAbout),
 EV_COMMAND(3100, CmActivate),
 EV_COMMAND(1101, CmMnuDisk),
 EV_COMMAND(1102, CmMnuInstr),
 EV_COMMAND(1100, CmMnuExit),
 EV_COMMAND(2103, CmLstNew),
 EV_COMMAND(2104, CmLstOpen),
 EV_COMMAND(2105, CmLstSave),
 EV_COMMAND(2106, CmLstSaveAs),
END_RESPONSE_TABLE;

/******************************************************************************/
// TMainFrameWindow Interface
/******************************************************************************/

class TMainFrameWindow : public TFrameWindow
{
  public:
   TDiskFrameWindow *DiskWnd;
   TInstrFrameWindow *InstrWnd;

   TPopupMenu *cntxt;
   TMainFrameWindow (TWindow* parent, const char far *title = APP_TITLE,
                   TWindow *clientWnd = 0, BOOL shrinkToClient = true);
   ~TMainFrameWindow();

   virtual void SetupWindow();
   void EvMove(TPoint &clientOrigin);
   void AdjustMyPosition();
   void SetVisibleF(int d, int i);

   DECLARE_RESPONSE_TABLE(TMainFrameWindow);
};


DEFINE_RESPONSE_TABLE1(TMainFrameWindow, TFrameWindow)
 EV_WM_MOVE,
END_RESPONSE_TABLE;

/******************************************************************************/
// Globals
/******************************************************************************/
TInstrDialog *dlgID;
TDiskDialog *dlgDD;
TMainDialog *dlgMD;

TMainFrameWindow *main_frame;

/*
  ##############################################################################
   The Implemenations
  ##############################################################################

   TPrefDlg
   TInstrDialog
   TInstrFrameWindow
   TDiskDialog
   TDiskFrameWindow
   TMainDialog
   TMainFrameWindow
*/


/******************************************************************************/
// TPrefDlg Implementation
/******************************************************************************/

TPrefDlg::TPrefDlg(TWindow* parent, TResId resId, TModule* module)
 :TDialog(parent, resId, module)
 {
  cb_Quality   = new TComboBox(this, 100);
  cb_Mixrate   = new TComboBox(this, 101);
  cb_Filter    = new TComboBox(this, 102);
  cb_BSizes    = new TComboBox(this, 103);
  cb_BBuffers  = new TComboBox(this, 104);
  cb_LoadNames = new TCheckBox(this, 105);
  cb_BadFiles  = new TCheckBox(this, 106);
  cb_IconTNA   = new TCheckBox(this, 107);
  cb_PlayBW    = new TCheckBox(this, 108);
  st_Important = new TStatic  (this, 200);

  f = new TFont("Helv", 16, 0, 0, 0, FW_BOLD);
 }

TPrefDlg::~TPrefDlg()
 {
  delete f;
 }

void TPrefDlg::SetupWindow()
 {
  TDialog::SetupWindow();
  SetupGUI();
 }

void TPrefDlg::CmOk()
 {
  MACfg.i_QualityIndex = cb_Quality->GetSelIndex();
  MACfg.i_MixIndex = cb_Mixrate->GetSelIndex();
  MACfg.i_FilterIndex = cb_Filter->GetSelIndex();
  MACfg.i_BSizeIndex = cb_BSizes->GetSelIndex();
  MACfg.i_BCountIndex = cb_BBuffers->GetSelIndex();
  MACfg.b_LoadNames = cb_LoadNames->GetCheck();
  MACfg.b_WarnBadFile = cb_BadFiles->GetCheck();
  MACfg.b_IconTNA = cb_IconTNA->GetCheck();
  MACfg.b_PlayBackwards = cb_PlayBW->GetCheck();
  TDialog::CmOk();
 }

void TPrefDlg::SetupGUI()
 {
  INIT_QUALITYTABLE;
  INIT_MIXTABLE;
  INIT_BSIZETABLE;
  INIT_FILTERTABLE;
  INIT_BCOUNTTABLE;

  for (int i = 0; i <= CNVQUALITYCNT-1; i++)
   cb_Quality->AddString(CNVQUALITY(i));
  cb_Quality->SetSelIndex(MACfg.i_QualityIndex);

  for (int i = 0; i <= CNVMIXRATECNT-1; i++)
   cb_Mixrate->AddString(CNVMIXRATE(i));
  cb_Mixrate->SetSelIndex(MACfg.i_MixIndex);

  for (int i = 0; i <= CNVFILTERCNT-1; i++)
   cb_Filter->AddString(CNVFILTER(i));
  cb_Filter->SetSelIndex(MACfg.i_FilterIndex);

  for (int i = 0; i <= CNVBSIZECNT-1; i++)
   cb_BSizes->AddString(CNVBSIZE(i));
  cb_BSizes->SetSelIndex(MACfg.i_BSizeIndex);

  for (int i = 0; i <= CNVBCOUNTCNT-1; i++)
   cb_BBuffers->AddString(CNVBCOUNT(i));
  cb_BBuffers->SetSelIndex(MACfg.i_BCountIndex);

  cb_LoadNames->SetCheck(MACfg.b_LoadNames);
  cb_BadFiles->SetCheck(MACfg.b_WarnBadFile);
  cb_IconTNA->SetCheck(MACfg.b_IconTNA);
  cb_PlayBW->SetCheck(MACfg.b_PlayBackwards);

  st_Important->SetWindowFont(*f, 0);
 }

/******************************************************************************/
// TInstrDialog Implementation
/******************************************************************************/
TInstrDialog::TInstrDialog(TWindow* parent, TResId resId, TModule* module)
             :TDialog(parent,resId,module), f("Courier New", 16)
 {
  InstrList = new TListBox(this, 100);
  dlgID = this;
 }

void TInstrDialog::SetupWindow()
 {
  TDialog::SetupWindow();
  InstrList->SetWindowFont(f, true);
 }

void TInstrDialog::UpdateInstrList(MIDASmodule module)
 {
  InstrList->ClearList();
  if (module != 0)
   {
    MIDASmoduleInfo mi;
    MIDASgetModuleInfo(module, &mi);
    for (unsigned i = 0; i < mi.numInstruments; i++)
     {
      MIDASinstrumentInfo ii;
      MIDASgetInstrumentInfo(module, i, &ii);
      char in[64];
      OemToChar(&ii.instName[0], &in[0]);
      char s[256];
      sprintf(s, "[%03u] %s", i+1, &in[0]);
      InstrList->AddString(&s[0]);
     }
   }

 };

/******************************************************************************/
// TDiskDialog Implementation
/******************************************************************************/
TDiskDialog::TDiskDialog(TWindow* parent, TResId resId, TModule* module)
   : TDialog(parent,resId,module)
 {
   dlgDD = this;
   FileList = new TListWindow(this, 100);
   img = new TImageList(*GetApplication(), "BMP_MODULE", 16, 1, TColor(0, 130, 0), IMAGE_BITMAP, ILC_COLOR4);
   img->SetBkColor(TColor(255, 255, 255));
   cntxt_idx = -1;
 }

void TDiskDialog::SetupWindow()
 {
  TDialog::SetupWindow();
  SetupGUI();
  LoadModulelistWithCmdLineCheck();
//  LoadModulelist();
//  ProcessCmdLine();

  SetupNewModule(CMD_AUTOFIRST);
  TPopupMenu *m = new TPopupMenu(TMenu(*GetApplication(), "MNU_DISK"));
  SetMenuDefaultItem(*m, 2100, false);
  AssignContextMenu(m);
  DragAcceptFiles(true);
 }

void TDiskDialog::LoadModulelistWithCmdLineCheck()
 {
  int b = 1;
  if (_argc > 1)
   {
    char s[MAX_PATH];
    strcpy(s, _argv[1]);
    char *p = strrchr(s, '.');
    if (p)
     {
      p++;
      if (stricmp(p, STR_MPL) == 0)
       {
        if (!strrchr(s, '\\'))
         {
          char path[MAX_PATH];
          strcpy(path, "X:\\");
          path[0] = (char) ('A' + getdisk());
          getcurdir(0, &path[3]);
          strcat(path, s);
          strcpy(MACfg.fname, path);
         }
        else
         strcpy(MACfg.fname, s);
        b = 0;
       }
     }
   }

  LoadModulelist();
  if (b) ProcessCmdLine();
 }

void TDiskDialog::CleanupWindow()
 {
  TDialog::CleanupWindow();
  SaveModulelist();
 }

void TDiskDialog::UnloadModule()
 {
   dlgMD->StopModule(0);
   dlgMD->UnloadFile(0);
   dlgMD->ShowStuckInfo();
   dlgMD->ShowVarInfo();
   dlgID->UpdateInstrList(0);
   MACfg.JustPlaying = 0;
   return;
 }

int TDiskDialog::LoadModulelist(int)
 {
  FileList->DeleteAllItems();
  TListWindItem o(STR_OTHER, 0);
  o.SetImageIndex(2);
  FileList->InsertItem(o);

  ifstream f(MACfg.fname, ios::in | ios::nocreate);
  if (f.fail()) return 0;

  while (!f.eof())
   {
    char s[512];
    char fp[MAX_PATH+2];
    char filename[MAX_PATH];
    char filepath[MAX_PATH];
    char filetype[MAX_PATH];
    char mname[70];
    char *p;

    filename[0] = 0;
    filepath[0] = 0;
    filetype[0] = 0;
    mname[0] = 0;

    f.getline(s, sizeof(s));
    if (s[0] != 0)
     {
       char *m = strstr(s, "','");
       memset(m, 0, 1);
       m += 2;

       if (strlen(s) > 2)
        {
         strcpy(fp, &s[1]);

         p = strrchr(&fp[0], 92);
         if (p)
          {
           memset(p, 0, 1);
           strcpy(filepath, fp);
           p++;
           strcpy(filename, p);
           p = strchr(&p[0], 46);
           p++;
           strcpy(filetype, p);
          };
        };

       if (strlen(m) > 2)
        {
         p = strchr(m, 0);
         if (p) {p--; memset(p, 0, 1);};
         m++;
         strcpy(mname, m);
        }

       strlwr(filename);
       TListWindItem i(filename, 0);
       i.SetImageIndex(0);
       int idx =FileList->InsertItem(i);

       TListWindItem i2(mname, 1);
       FileList->SetItemText(idx, i2);

       TListWindItem i3(filetype, 2);
       FileList->SetItemText(idx, i3);

       TListWindItem i4(filepath, 3);
       FileList->SetItemText(idx, i4);
     }
  }

  f.rdbuf()->close();
  UpdateTitle();

  return 1;
 }

int TDiskDialog::SaveModulelist()
 {
  ofstream f(MACfg.fname, ios::out |ios::trunc);
  if (f.fail()) return 0;

  for (int i= 1; i <= FileList->GetItemCount()-1; i++)
   {
    char buffer[MAX_PATH];
    char s[512];

    TListWindItem lvi(buffer, 0, sizeof(buffer));
    lvi.SetIndex(i);
    lvi.SetSubItem(0);
    FileList->GetItem(lvi);
    if (strcmp(buffer, STR_OTHER))
     {
       strcpy(s, "'");

       lvi.SetSubItem(3);
       FileList->GetItem(lvi);
       strcat(strcat(s, buffer), "\\");

       lvi.SetSubItem(0);
       FileList->GetItem(lvi);
       strcat(strcat(s, buffer), "','");

       lvi.SetSubItem(1);
       FileList->GetItem(lvi);
       strcat(strcat(s, buffer), "'\n");

       f.write(s, strlen(s));
     }

   }

  f.rdbuf()->close();
  return 1;
 }

int TDiskDialog::SetupGUI()
 {
  int n = (FileList->Attr.W-2-GetSystemMetrics(SM_CXVSCROLL)) / 4;
  FileList->InsertColumn(0, TListWindColumn(STR_FNAME, n));
  FileList->InsertColumn(1, TListWindColumn(STR_MNAME, n));
  FileList->InsertColumn(2, TListWindColumn(STR_TYPE, n));
  FileList->InsertColumn(3, TListWindColumn(STR_PATH, FileList->Attr.W-(3*n)-4));

  FileList->SetImageList(*img, TListWindow::Small);

  return 1;
 }

void TDiskDialog::EvModuleDblclk()
 {
   TLwHitTestInfo pt;
   ::GetCursorPos(&pt.pt);
   ::ScreenToClient(FileList->HWindow, &pt.pt);
   int idx = FileList->HitTest(pt);

   if (idx >= 1)
    PlayIndex(idx);

   if (idx == 0)
    AddNewFiles();
 }

int TDiskDialog::AddFile(char *p, char *fn)
 {
  char descr[256];
  strcpy(descr, STR_NONAMELOADED);

  char fullpath[MAX_PATH];
  fullpath[0] = 0;
  if ((p) && ((*p) != 0)) strcat(strcpy(fullpath, p), "\\");
  strcat(fullpath, fn);

  if (FileExists(fullpath))
   {
    strlwr(fn);
    TLwFindInfo fi;
    fi.SetString(fn);
    int idx = FileList->FindItem(idx, &fi);

    if (idx == -1)
     {
      if (MACfg.b_LoadNames)
       {
         MIDASmodule m = MIDASloadModule(fullpath);
         if (m)
          {
           MIDASmoduleInfo mi;
           MIDASgetModuleInfo(m, &mi);
           strcpy(descr, mi.songName);
           if (descr[0] == 0) strcpy(descr, STR_NONAME);
           MIDASfreeModule(m);
          }
         else
          {
           if (MACfg.b_WarnBadFile) MessageBox(STR_BADMODULE, STR_MODALITY, MB_ICONERROR);
           return 0;
          }
       }

      TListWindItem i(fn, 0);
      idx = FileList->InsertItem(i);

      TListWindItem i2(descr, 1);
      i2.SetImageIndex(0);
      FileList->SetItemText(idx, i2);

      char *p2 = strchr(fn, 46);
      if (p) p2++; else p2 = STR_NOTYPE;

      TListWindItem i3(p2, 2);
      FileList->SetItemText(idx, i3);

      TListWindItem i4(p, 3);
      FileList->SetItemText(idx, i4);
      return 1;
     }
   }
  return 0;
 }

int TDiskDialog::AddNewFiles()

 {
   TFileOpenDialog::TData d(OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_ALLOWMULTISELECT | OFN_EXPLORER,
                            STR_MODTYPES, 0, "", "MOD");

   char s[8000], *oldp, *p;

   oldp = d.FileName;
   d.FileName = &s[0];
   d.MaxPath = sizeof(s);

   if (TFileOpenDialog(this, d).Execute() == IDOK)
    {
     SetCursor(GetApplication(), IDC_WAIT);
     p = strchr(d.FileName, 0);
     p++;
     if (*p == NULL)
      {
        p = strrchr(d.FileName, 92);
        if (p)
         {
          (*p) = 0;
          p++;
          AddFile(d.FileName, p);
         }
        else
         AddFile(0, d.FileName);
      }
     else
      {
       while (*p != NULL)
        {
         AddFile(d.FileName, p);
         p = strchr(p, 0);
         p++;
        }
      }
     SetCursor(GetApplication(), IDC_ARROW);
    }
   d.FileName = oldp;
   return 1;
 }

int TDiskDialog::SetupNewModule(int cmd)
 {
  if (FileList->GetItemCount() == 1)
   {
    UnloadModule();
    return 1;
   }

  int i = MACfg.JustPlaying;
  switch (cmd)
   {
    case CMD_AUTO, CMD_AUTOFIRST :
     if ((cmd == CMD_AUTOFIRST) && (MACfg.JustPlaying >= 1)) break;
     if (MACfg.ShuffleFlag)
      i = random(FileList->GetItemCount()-1)+1;
     else
      if (!MACfg.b_PlayBackwards)
       i = MACfg.JustPlaying+1;
      else
       i = MACfg.JustPlaying-1;
    break;
    case CMD_NEXT, CMD_NEXTNOREPEAT :
     i = MACfg.JustPlaying+1;
    break;
    case CMD_PREVIOUS :
     i = MACfg.JustPlaying-1;
    break;
   }

  if (MACfg.Repeat2Flag)
   {
    if (i < 1) i = FileList->GetItemCount()-1;
    if (i > FileList->GetItemCount()-1) i = 1;
   }
  else
   {
    if (i < 1) i = 1;
    if (i > FileList->GetItemCount()-1) i = FileList->GetItemCount()-1;
   }

  if ((cmd == CMD_NEXTNOREPEAT) && (MACfg.JustPlaying == i))
   UnloadModule();
  else
   PlayIndex(i);

  return cmd;
 }

int TDiskDialog::PlayIndex(int idx)
 {
  char buffer[512];
  char fn[MAX_PATH];

  TListWindItem lvi(buffer, 0, sizeof(buffer));
  lvi.SetIndex(idx);
  if (FileList->GetItem(lvi))
   {
    strcpy(fn, buffer);

    lvi.SetSubItem(3);
    FileList->GetItem(lvi);
    strcat(strcat(buffer, "\\"), fn);

    dlgMD->StopModule(0);
    dlgMD->UnloadFile(0);
    MarkModule(MACfg.JustPlaying, 0);

    strcpy(dlgMD->fname, buffer);
    if (dlgMD->LoadFile())
     {
      dlgMD->ShowStuckInfo();
      dlgMD->ShowVarInfo();
      dlgID->UpdateInstrList(dlgMD->module);
      dlgMD->PlayModule(0);

      lvi.SetSubItem(1);
      if (dlgMD->imodule.songName[0] != 0)
       strcpy(buffer, dlgMD->imodule.songName);
      else
       strcpy(buffer, STR_NONAME);
      FileList->SetItem(lvi);
      MarkModule(idx, 1);
      MACfg.JustPlaying = idx;
      FileList->EnsureVisible(MACfg.JustPlaying, 0);
     }
    else
     {
      MessageBox(STR_BADMODULE, STR_MODALITY, MB_ICONERROR);
      dlgID->InstrList->ClearList();
      dlgMD->ShowStuckInfo();
      dlgMD->ShowVarInfo();
     }
    return idx;
  }
  else
   return 0;
 }

int TDiskDialog::MarkModule(int idx, int b)
 {
  TListWindItem lvi;
  lvi.SetIndex(idx);
  lvi.SetImageIndex(!b ? 0 : 1);
  FileList->SetItem(lvi);
  return idx;
 }

void TDiskDialog::EvContextMenu(HWND child, int x, int y)
 {
  TLwHitTestInfo pt;
  pt.pt.x = x;
  pt.pt.y = y;
  ::ScreenToClient(FileList->HWindow, &pt.pt);
  cntxt_idx = FileList->HitTest(pt);

  TPopupMenu *m = GetContextMenu();
  m->EnableMenuItem(2100, MF_BYCOMMAND | ((cntxt_idx >= 1) ? MF_ENABLED : MF_GRAYED));
  m->EnableMenuItem(2102, MF_BYCOMMAND | ((cntxt_idx >= 1) ? MF_ENABLED : MF_GRAYED));

  TDialog::EvContextMenu(child, x, y);
 }

void TDiskDialog::CmPlay()
 {
  PlayIndex(cntxt_idx);
 }

void TDiskDialog::CmDelete()
 {
  if (cntxt_idx == MACfg.JustPlaying)
   SetupNewModule(CMD_NEXTNOREPEAT);

  if (cntxt_idx != MACfg.JustPlaying)
   {
    FileList->DeleteAnItem(cntxt_idx);
    if (MACfg.JustPlaying > cntxt_idx) MACfg.JustPlaying--;
   }
 }

void TDiskDialog::CmAddFile()
 {
   AddNewFiles();
 }

void TDiskDialog::CmLstNew()
 {
  CmLstOpen();
 }

void TDiskDialog::CmLstOpen()
 {
   TFileOpenDialog::TData d(OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_EXPLORER,
                            STR_MPLTYPES, 0, "", "MPL");

   char s[MAX_PATH], *oldp;

   oldp = d.FileName;
   d.FileName = &s[0];
   d.MaxPath = sizeof(s);
   if (TFileOpenDialog(this, d).Execute() == IDOK)
    {
     SaveModulelist();
     UnloadModule();
     strcpy(MACfg.fname, d.FileName);
     LoadModulelist();
     MACfg.JustPlaying = 0;
     SetupNewModule(CMD_AUTO);
    }
   d.FileName = oldp;
 }

void TDiskDialog::CmLstSave()
 {
  SaveModulelist();
 }

void TDiskDialog::CmLstSaveAs()
 {
   TFileSaveDialog::TData d(OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_EXPLORER,
                            STR_MPLTYPES, 0, "", "MPL");

   char s[MAX_PATH], *oldp;

   oldp = d.FileName;
   d.FileName = &s[0];
   d.MaxPath = sizeof(s);
   if (TFileSaveDialog(this, d).Execute() == IDOK)
    {
     strcpy(MACfg.fname, d.FileName);
     SaveModulelist();
    }
   d.FileName = oldp;
 }

void TDiskDialog::EvDropFiles(TDropInfo dropInfo)
 {
  for (int i = dropInfo.DragQueryFileCount()-1; i >= 0; i--)
   {
    char s[MAX_PATH];
    dropInfo.DragQueryFile(i, s, sizeof(s));
    char* p = strrchr(s, 92);
    *p = 0;
    p ++;
    AddFile(s, p);
   }
 }

void TDiskDialog::ProcessCmdLine()
 {
  for (int i = 1; i <= _argc-1; i++)
   {
    char s[MAX_PATH];
    strcpy(s, _argv[i]);
    char *p = strrchr(s, 92);
    if (p)
     {
      *p = 0; p++;
      AddFile(s, p);
     }
    else
     {
      char path[MAX_PATH];
      strcpy(path, "X:\\");
      path[0] = (char) ('A' + getdisk());
      getcurdir(0, &path[3]);
      AddFile(path, s);
     }
   }
 }

void TDiskDialog::UpdateTitle()
 {
  char s[255], s2[MAX_PATH];
  strcat(strcpy(s, APP_TITLE), " [");
  strcpy(s2, MACfg.fname);

  char *p = strrchr(s2, 92);

  if (p)
   p++;
  else
   p = &s2[0];

  char *p2 = strrchr(p, 46);
  if (p2) *p2 = 0;

  strcat(s, p);

  strcat(s, "]");
  main_frame->SetCaption(s);
 }

int TDiskDialog::FileExists(char *p)
 {
  ffblk t;
  int i = findfirst(p, &t, FA_NORMAL | FA_RDONLY | FA_HIDDEN);

  return (i != -1);
 }

/******************************************************************************/
// TMainDialog Implementation
/******************************************************************************/
TMainDialog::TMainDialog(TWindow* parent, TResId resId, TModule* module)
            :TDialog(parent,resId,module), f("Times New Roman", 28, 0, 0, 0, FW_NORMAL, true)
 {
   st_title = new TStatic(this, 100, 256);
   st_fname = new TStatic(this, 101, 256);
   st_type  = new TStatic(this, 102, 256);
   st_arr   = new TStatic(this, 103, 256);
   st_pat   = new TStatic(this, 104, 256);
   st_chan  = new TStatic(this, 105, 256);
   st_instr = new TStatic(this, 106, 256);
   sl_pos   = new THSlider(this, 115);
   sl_vol   = new TVSlider(this, 117);
   bt_play  = new TButton(this, 107);
   bt_stop  = new TButton(this, 108);
   bt_rewind  = new TButton(this, 111);
   bt_forward = new TButton(this, 112);
   cb_repeat  = new TCheckBox(this, 118);
   cb_repeat2 = new TCheckBox(this, 122);
   cb_shuffle = new TCheckBox(this, 119);
   cb_disk    = new TCheckBox(this, 120);
   cb_instr   = new TCheckBox(this, 121);

   dlgMD = this;

   ms = nothing;
   fname[0] = 0;
   setmem(&pstatus, sizeof(pstatus), 0);
   setmem(&imodule, sizeof(imodule), 0);

   lastpos = -1;

   icoSmall = new TIcon(*GetApplication(), "ICO_SMALL");
   icoPlay = new TIcon(*GetApplication(), "ICO_PLAY");
   icoNothing = new TIcon(*GetApplication(), "ICO_NOTHING");

   hMailslot = 0;

   pop = new TPopupMenu(TMenu(*GetApplication(), "MNU_TNA"));
   SetMenuDefaultItem(*pop, 3100, false);
 }

TMainDialog::~TMainDialog()
 {
   delete icoSmall;
   delete icoPlay;
   delete icoNothing;
   delete pop;
 }

void TMainDialog::CmPlay()
 {
   PlayModule(sl_pos->GetPosition());
 };

void TMainDialog::CmStop()
 {
   StopModule();
 }

void TMainDialog::SetupWindow()
 {
  TDialog::SetupWindow();
  SetupGUI();
  InitMIDAS();

  NOTIFYICONDATA nid;
  nid.cbSize = sizeof(nid);
  nid.hWnd = HWindow;
  nid.uID = 4711;
  nid.uFlags = NIF_ICON | NIF_MESSAGE;
  nid.hIcon = ((ms == playing) ? icoPlay->GetHandle() : icoSmall->GetHandle());
  nid.uCallbackMessage = WM_TNANOTIFY;
  Shell_NotifyIcon(NIM_ADD, &nid);

  SetTimer(4711, 100);

  InitMailSlot();
  PostMessage(WM_AFTERINIT, 0, 0);
 }

void TMainDialog::CleanupWindow()
 {
  TDialog::CleanupWindow();
  KillTimer(4711);
  CloseMailSlot();
  StopModule();
  UnloadFile();
  DoneMIDAS();

  NOTIFYICONDATA nid;
  nid.cbSize = sizeof(nid);
  nid.hWnd = HWindow;
  nid.uID = 4711;
  Shell_NotifyIcon(NIM_DELETE, &nid);
 }

int TMainDialog::LoadFile()
 {
  if (ms == nothing)
   {
    module = MIDASloadModule(fname);
    if (module)
     {
      MIDASgetModuleInfo(module, &imodule);
      setmem(&pstatus, sizeof(pstatus), 0);
      ms = loaded;
      UpdateState();
      return 1;
     }
   }
  return 0;
 }

int TMainDialog::UnloadFile(int upd)
 {
   if (ms == loaded)
    {
     if (MIDASfreeModule(module))
      {
        ms = nothing;
        if (upd)
         {
          UpdateState();
          ShowStuckInfo();
         }
        return 1;
      }
    }
   return 0;
 }
int TMainDialog::PlayModule(unsigned n)
 {
  if (ms == loaded)
   {
    pmodule = MIDASplayModuleSection(module, n, imodule.songLength-1, 0, 1);
    if (pmodule)
     {
       MIDASsetMusicVolume(pmodule, MACfg.volume);
       ms = playing;
       UpdateState();
       MIDASgetPlayStatus(pmodule, &pstatus);
       return 1;
     }
   }
  return 0;
 }

int TMainDialog::StopModule(int upd)
 {
   if (ms == playing)
    {
      if (MIDASstopModule(pmodule))
       {
        ms = loaded;
        if (upd) UpdateState();
        return 1;
       }
    }
   return 0;
 }

int TMainDialog::ShowStuckInfo()
 {
  if (ms != nothing)
   {
    char *t;
    char s[255];

    OemToChar(&imodule.songName[0], &s[0]);
    if (imodule.songName[0] == 0)
     strcpy(s, STR_NONAME);

    st_title->SetText(s);

    NOTIFYICONDATA nid; nid.cbSize = sizeof(nid);
    nid.hWnd = HWindow;
    nid.uID = 4711;
    nid.uFlags = NIF_ICON;
    if (ms == playing)
     nid.hIcon = icoPlay->GetHandle();
    else
     nid.hIcon = icoSmall->GetHandle();
    Shell_NotifyIcon(NIM_MODIFY, &nid);

    t = strrchr(fname, 92);  //"\"
    if (t) t++; else t = &fname[0];
    st_fname->SetText(t);

    strcpy(s, STR_NOTYPE);

    t = strrchr(fname, 46);  // "."
    if (t) t++; else t = &s[0];
    st_type->SetText(t);

    itoa(imodule.songLength, s, 10);
    st_arr->SetText(s);
    itoa(imodule.numPatterns, s, 10);
    st_pat->SetText(s);
    itoa(imodule.numChannels, s, 10);
    st_chan->SetText(s);
    itoa(imodule.numInstruments, s, 10);
    st_instr->SetText(s);

    sl_pos->SetRange(0, imodule.songLength-1);

    return 1;
   }
  else
   {
    st_instr->SetText(STR_NOTAVAILABLE);
    st_arr->SetText(STR_NOTAVAILABLE);
    st_pat->SetText(STR_NOTAVAILABLE);
    st_chan->SetText(STR_NOTAVAILABLE);
    st_type->SetText(STR_NOTAVAILABLE);
    st_fname->SetText(STR_NOTAVAILABLE);
    st_title->SetText(STR_NOTAVAILABLE);
    sl_pos->SetRange(0, 0);

    return 1;
   }
 }

int TMainDialog::ShowVarInfo()
 {
   if ((lastpos != pstatus.position) && (ms == playing))
    {
      sl_pos->SetPosition(pstatus.position);

      char s[255], s2[255];
      itoa(imodule.songLength, s, 10);
      itoa(pstatus.position+1, s2, 10);
      strcat(strcat(s2, "/"), s);
      st_arr->SetText(s2);
      lastpos = pstatus.position;

      OemToChar(&imodule.songName[0], &s[0]);
      if (imodule.songName[0] == 0)
       strcpy(s, STR_NONAME);

      sprintf(s2, "%s.%u/%u", &s, pstatus.position, imodule.songLength-1);
      NOTIFYICONDATA nid; nid.cbSize = sizeof(nid);
      nid.hWnd = HWindow;
      nid.uID = 4711;
      nid.uFlags = NIF_TIP;
      strcpy(nid.szTip, s2);
      Shell_NotifyIcon(NIM_MODIFY, &nid);

      return 1;
    }
   return 0;
 }

int TMainDialog::SetupOptions()
 {
   StopModule();
   UnloadFile();
   MIDASstopBackgroundPlay();
   MIDASclose();
   INIT_MIXTABLE;
   MIDASsetOption(MIDAS_OPTION_MIXRATE, GETMIXRATE);
   MIDASsetOption(MIDAS_OPTION_OUTPUTMODE, GETQUALITY);
   MIDASsetOption(MIDAS_OPTION_FILTER_MODE, GETFILTER);
   MIDASsetOption(MIDAS_OPTION_MIXBUFLEN, GETBSIZE);
   MIDASsetOption(MIDAS_OPTION_MIXBUFBLOCKS, GETBCOUNT);
   MIDASinit();
   MIDASstartBackgroundPlay(POLLRATE);
   LoadFile();
   PlayModule(sl_pos->GetPosition());
   ShowStuckInfo();
   return 1;
 }

int TMainDialog::InitMIDAS()
 {
  INIT_MIXTABLE;
  MIDASsetOption(MIDAS_OPTION_MIXRATE, GETMIXRATE);
  MIDASsetOption(MIDAS_OPTION_OUTPUTMODE, GETQUALITY);
  MIDASsetOption(MIDAS_OPTION_FILTER_MODE, GETFILTER);
  MIDASsetOption(MIDAS_OPTION_MIXBUFLEN, GETBSIZE);
  MIDASsetOption(MIDAS_OPTION_MIXBUFBLOCKS, GETBCOUNT);

  if (MIDASinit())
   {
    if (MIDASstartBackgroundPlay(POLLRATE)) return 1;
   }
  return 0;
 }

int TMainDialog::DoneMIDAS()
 {
   MIDASstopBackgroundPlay();
   return MIDASclose();
 }


void TMainDialog::EvTimer(uint timerId)
 {
   if (timerId == 4711)
    {
      ProcessMailSlot();
      if (ms == playing)
       {
         MIDASgetPlayStatus(pmodule, &pstatus);
         if (pstatus.songLoopCount)
          {
           StopModule();
           if (MACfg.RepeatFlag)
            {
             sl_pos->SetPosition(0);
             PlayModule(0);
            }
           else
            {
             dlgDD->SetupNewModule();
            }
          }
         ShowVarInfo();
       }
      else
       {
         if (lastpos != -1)
          {
            ShowStuckInfo();
            lastpos = -1;
          }
       }
    }
 }

int TMainDialog::SetupGUI()
 {
  st_title->SetWindowFont(f, true);

  sl_pos->SetPageMagnitude(1);
  sl_pos->SetLineMagnitude(1);
  sl_pos->SetRuler(1, true);

  sl_vol->SetPageMagnitude(8);
  sl_vol->SetLineMagnitude(1);
  sl_vol->SetRuler(1, true);
  sl_vol->SetRange(0, 64);

  sl_vol->SetPosition(64-MACfg.volume);

  cb_repeat->SetCheck(MACfg.RepeatFlag);
  cb_shuffle->SetCheck(MACfg.ShuffleFlag);
  cb_repeat2->SetCheck(MACfg.Repeat2Flag);
  cb_disk->SetCheck(MACfg.ShowDisk);
  cb_instr->SetCheck(MACfg.ShowInstr);

  return 1;
 }

void TMainDialog::EvTrack()
 {
   if (ms == playing)
    {
      MIDASsetPosition(pmodule, sl_pos->GetPosition());
      ShowVarInfo();
    }
 }

void TMainDialog::EvVol()
 {
   MACfg.volume = 64-sl_vol->GetPosition();
   if (ms == playing) MIDASsetMusicVolume(pmodule, MACfg.volume);
 }

int TMainDialog::UpdateState()
 {
   bt_stop->EnableWindow(ms == playing);
   bt_play->EnableWindow(ms == loaded);
   bt_forward->EnableWindow(ms != nothing);
   bt_rewind->EnableWindow(ms != nothing);
   sl_pos->EnableWindow(ms != nothing);

   NOTIFYICONDATA nid;
   nid.cbSize = sizeof(nid);
   nid.hWnd = HWindow;
   nid.uID = 4711;
   nid.uFlags = NIF_ICON;
   nid.hIcon = ((ms == playing) ? icoPlay->GetHandle() : ((ms == loaded) ? icoSmall->GetHandle() : icoNothing->GetHandle()));
   Shell_NotifyIcon(NIM_MODIFY, &nid);

   return 1;
 }

void TMainDialog::CmForward()
 {
  if (ms == loaded)
   {
     unsigned n = sl_pos->GetPosition();
     if (n < imodule.songLength-1) n++;
     sl_pos->SetPosition(n);
   }
  if (ms == playing)
   {
    MIDASgetPlayStatus(pmodule, &pstatus);
    unsigned p = pstatus.position;
    if (p < (imodule.songLength-1)) p++;
    MIDASsetPosition(pmodule, p);
    ShowVarInfo();
   }
 }

void TMainDialog::CmRewind()
 {
  if (ms == loaded)
   {
     unsigned n = sl_pos->GetPosition();
     if (n > 0) n--;
     sl_pos->SetPosition(n);
   }
  if (ms == playing)
   {
    MIDASgetPlayStatus(pmodule, &pstatus);
    unsigned p = pstatus.position;
    if (p > 0) p--;
    MIDASsetPosition(pmodule, p);
    ShowVarInfo();
   }
 }

LPARAM TMainDialog::EvAfterInit(WPARAM, LPARAM)
 {
  dlgID->UpdateInstrList(module);
  UpdateFlags();
  return 0;
 }

void TMainDialog::CmRepeat()
 {
  MACfg.RepeatFlag = (cb_repeat->GetCheck() != 0);
  UpdateFlags();
 }

void TMainDialog::CmShuffle()
 {
  MACfg.ShuffleFlag = (cb_shuffle->GetCheck() != 0);
  UpdateFlags();
 }

 void TMainDialog::CmRepeat2()
 {
  MACfg.Repeat2Flag = (cb_repeat2->GetCheck() != 0);
  UpdateFlags();
 }

void TMainDialog::CmDisk()
 {
   main_frame->SetVisibleF((cb_disk->GetCheck() != 0), MACfg.ShowInstr);
   UpdateFlags();
 }

void TMainDialog::CmInstr()
 {
   main_frame->SetVisibleF(MACfg.ShowDisk, (cb_instr->GetCheck() != 0));
   UpdateFlags();
 }

LRESULT TMainDialog::EvTNANotify(WPARAM w, LPARAM l)
 {
  if (w == 4711)
   {
    switch (LOWORD(l))
     {
      case WM_LBUTTONDBLCLK:
       CmActivate();
      break;
      case WM_RBUTTONUP:
       POINT p;
       ::GetCursorPos(&p);
       pop->TrackPopupMenu(TPM_RIGHTBUTTON, p.x, p.y, 0, *Parent);
      break;
     }
   }
  return 1;
 }

void TMainDialog::ProcessMailSlot()
 {
  return;
  DWORD cbm, cm = 0;
  DWORD to = 0;
  bool fResult = GetMailslotInfo(hMailslot, 0, &cbm, &cm, &to);

  while (fResult && (cm != MAILSLOT_NO_MESSAGE))
   {
    char buf[MAX_PATH];
    buf[0] = 0;
    fResult = ReadFile(hMailslot, buf, cbm, &cbm, 0);

    if (strcmp(buf, STR_ACTIVATE) == 0)
     SetForegroundWindow(HWindow);
    else
     {
      if (fResult)
       {
        char *p = strrchr(buf, 92);
        if (p)
         {
          *p = 0;
          p++;
          dlgDD->AddFile(buf, p);
         }
        else
         dlgDD->AddFile(0, buf);
       }
     }
    GetMailslotInfo(hMailslot, 0, &cbm, &cm, 0);
   }
 }

void TMainDialog::InitMailSlot()
 {
   hMailslot = CreateMailslot(APP_MAILSLOT, 0, 0, 0);
   if (hMailslot == INVALID_HANDLE_VALUE) hMailslot = 0;
 }

void TMainDialog::CloseMailSlot()
 {
   CloseHandle(hMailslot);
 }

void TMainDialog::CmPref()
 {
  TPrefDlg *dlg = new TPrefDlg(this, "DLG_CONFIG");
  if (dlg->Execute() == 1)
   SetupOptions();
  delete dlg;
 }

void TMainDialog::CmFForward()
 {
  dlgDD->SetupNewModule(CMD_NEXT);
 }

void TMainDialog::CmFRewind()
 {
  dlgDD->SetupNewModule(CMD_PREVIOUS);
 }

void TMainDialog::CmFAuto()
 {
  dlgDD->SetupNewModule(CMD_AUTO);
 }

void TMainDialog::CmMnuShuffle()
 {
  MACfg.ShuffleFlag = !MACfg.ShuffleFlag;
  cb_shuffle->SetCheck(MACfg.ShuffleFlag);
  UpdateFlags();
 }

void TMainDialog::CmMnuRepeat1()
 {
  MACfg.RepeatFlag = !MACfg.RepeatFlag;
  cb_repeat->SetCheck(MACfg.RepeatFlag);
  UpdateFlags();
 }

void TMainDialog::CmMnuRepeat()
 {
  MACfg.Repeat2Flag = !MACfg.Repeat2Flag;
  cb_repeat2->SetCheck(MACfg.Repeat2Flag);
  UpdateFlags();
 }

void ChkMnu(TMenu &m)
 {
  m.CheckMenuItem(3104, MF_BYCOMMAND | (dlgMD->cb_shuffle->GetCheck()!= 0 ? MF_CHECKED : 0));
  m.CheckMenuItem(3105, MF_BYCOMMAND | (dlgMD->cb_repeat->GetCheck()!= 0 ? MF_CHECKED : 0));
  m.CheckMenuItem(3108, MF_BYCOMMAND | (dlgMD->cb_repeat2->GetCheck()!= 0 ? MF_CHECKED : 0));
  m.CheckMenuItem(1101, MF_BYCOMMAND | (dlgMD->cb_disk->GetCheck()!= 0 ? MF_CHECKED : 0));
  m.CheckMenuItem(1102, MF_BYCOMMAND | (dlgMD->cb_instr->GetCheck()!= 0 ? MF_CHECKED : 0));
 }

void TMainDialog::UpdateFlags()
 {
  ChkMnu(*pop);
  ChkMnu(TMenu(TMenu(Parent->GetMenu()).GetSubMenu(1)));
  ChkMnu(TMenu(TMenu(Parent->GetMenu()).GetSubMenu(2)));
  ChkMnu(TMenu(main_frame->cntxt->GetSubMenu(1)));
  ChkMnu(TMenu(main_frame->cntxt->GetSubMenu(2)));
 }

void TMainDialog::CmAbout()
 {
  TDialog *dlg = new TDialog(this, "DLG_INFO");
  dlg->Execute();
  delete dlg;
 }

void TMainDialog::CmActivate()
 {
  main_frame->ShowWindow(SW_SHOW);
  SetForegroundWindow(*main_frame);
  main_frame->ShowWindow(SW_RESTORE);
 }

void TMainDialog::CmMnuDisk()
 {
  MACfg.ShowDisk = !MACfg.ShowDisk;
  cb_disk->SetCheck(MACfg.ShowDisk);
  UpdateFlags();
  main_frame->SetVisibleF((cb_disk->GetCheck() != 0), MACfg.ShowInstr);
 }

void TMainDialog::CmMnuInstr()
 {
  MACfg.ShowInstr = !MACfg.ShowInstr;
  cb_instr->SetCheck(MACfg.ShowInstr);
  UpdateFlags();
  main_frame->SetVisibleF(MACfg.ShowDisk, (cb_instr->GetCheck() != 0));
 }

void TMainDialog::CmMnuExit()
 {
  PostQuitMessage(0);
 }

void TMainDialog::CmLstNew()
 {
  dlgDD->CmLstNew();
 }

void TMainDialog::CmLstOpen()
 {
  dlgDD->CmLstOpen();
 }

void TMainDialog::CmLstSave()
 {
  dlgDD->CmLstSave();
 }

void TMainDialog::CmLstSaveAs()
 {
  dlgDD->CmLstSaveAs();
 }

void TMainDialog::CmMnuHelp()
 {
  WinHelp("modality.hlp", HELP_INDEX, 0);
 }

/******************************************************************************/
// TMainFrameWindow Implementation
/******************************************************************************/
TMainFrameWindow::TMainFrameWindow (TWindow* parent, const char far *title,
                   TWindow *clientWnd, BOOL shrinkToClient)
                  :TFrameWindow (parent, title, clientWnd, shrinkToClient)
 {
  Attr.Style &= ~WS_THICKFRAME &~WS_MAXIMIZEBOX ;
  Attr.ExStyle |= WS_EX_DLGMODALFRAME | WS_EX_APPWINDOW;
  main_frame = this;
  SetIcon(GetApplication(),"ICO_MAIN");
  SetIconSm(GetApplication(),"ICO_SMALL");
  cntxt = new TPopupMenu(TMenu(*GetApplication(), "MNU_MAIN"));
 }

TMainFrameWindow::~TMainFrameWindow()
 {
//  if (cntxt) delete cntxt;
 }

void TMainFrameWindow::SetupWindow()
 {
   TFrameWindow::SetupWindow();

   DiskWnd = new TDiskFrameWindow(this, NULL, new TDiskDialog(0));
   InstrWnd = new TInstrFrameWindow(this, NULL, new TInstrDialog(0));
   DiskWnd->Create();
   InstrWnd->Create();
   AdjustMyPosition();

   AssignContextMenu(cntxt);
   AssignMenu("MNU_MAIN");
 };

void TMainFrameWindow::EvMove(TPoint &clientOrigin)
 {
   TFrameWindow::EvMove(clientOrigin);
   AdjustMyPosition();
 }

#define YSPACE 2
void TMainFrameWindow::AdjustMyPosition()
 {
   if (!IsIconic())
    {
      if (MACfg.ShowDisk)
       DiskWnd->SetWindowPos(0, Attr.X, Attr.Y+Attr.H+YSPACE, Attr.W, DiskWnd->Attr.H, SWP_NOZORDER | SWP_SHOWWINDOW | SWP_NOACTIVATE);
      else
       DiskWnd->SetWindowPos(0, 0, 0, 0, 0, SWP_NOZORDER | SWP_NOSIZE | SWP_NOMOVE | SWP_HIDEWINDOW | SWP_NOACTIVATE);

      if (MACfg.ShowInstr)
       {
         if (MACfg.ShowDisk)
          InstrWnd->SetWindowPos(0, Attr.X, Attr.Y+Attr.H+DiskWnd->Attr.H+2*YSPACE, Attr.W, InstrWnd->Attr.H, SWP_NOZORDER | SWP_SHOWWINDOW | SWP_NOACTIVATE);
         else
          InstrWnd->SetWindowPos(0, Attr.X, Attr.Y+Attr.H+YSPACE, Attr.W, InstrWnd->Attr.H, SWP_NOZORDER | SWP_SHOWWINDOW | SWP_NOACTIVATE);
       }
      else
       InstrWnd->SetWindowPos(0, 0, 0, 0, 0, SWP_NOZORDER | SWP_NOSIZE | SWP_NOMOVE | SWP_HIDEWINDOW | SWP_NOACTIVATE);
    }
   else
   {
    if (MACfg.b_IconTNA) ShowWindow(SW_HIDE);
   }
 }

void TMainFrameWindow::SetVisibleF(int d, int i)
 {
  MACfg.ShowDisk = d;
  MACfg.ShowInstr = i;
  AdjustMyPosition();
 }

/******************************************************************************/
// TMainApp Interface+Implementation
/******************************************************************************/
class TMainApp : public TApplication
{
  public:
   TMainApp(const char far* name = 0, TModule*& gModule = ::Module, TAppDictionary* appDict = 0);
   ~TMainApp();

   ATOM hAtom;
   void InitMainWindow()
    {
     SetMainWindow(new TMainFrameWindow(0, APP_TITLE, new TMainDialog(0)));
     EnableCtl3d();
    }

   void SendArgs();
};

TMainApp::TMainApp(const char far* name, TModule*& gModule, TAppDictionary* appDict)
 :TApplication(name, gModule, appDict)
 {

  // Only to destroy rests of older instances
  /****************************************************************************/
  GlobalDeleteAtom(GlobalFindAtom(APP_ATOM));
  /****************************************************************************/


  if (GlobalFindAtom(APP_ATOM)) SendArgs();
  hAtom = GlobalAddAtom(APP_ATOM);

  randomize();
  MIDASstartup();
  CFG_INIT;
  CFG_LOAD;
 }

TMainApp::~TMainApp()
 {
  CFG_SAVE;
  GlobalDeleteAtom(hAtom);
 }

void TMainApp::SendArgs()
 {
  HANDLE hFile;

  hFile = CreateFile(APP_MAILSLOT, GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if (GetLastError() == 0)
   {
    DWORD cbWritten;

    for (int i = 1; i <= _argc-1; i++)
     WriteFile(hFile, _argv[i], lstrlen(_argv[i]) + 1, &cbWritten, 0);

    if (_argc <= 1)
     {
      char s[255];
      strcpy(s, STR_ACTIVATE);
      WriteFile(hFile, s, strlen(s)+1, &cbWritten, 0);
     }

    CloseHandle(hFile);
   }
  exit(EXIT_SUCCESS);
 }

int OwlMain(int, char*[])
{
 return  TMainApp(APP_TITLE).Run();
}
